/**
 * Painter
 * by Andres Colubri. February 2008.
 * 
 * Creates a particle system on the GPU that paints the screen according
 * to the underlying image.
 * This version uses Geometry shaders to accelerate rendering.
 *
 */
 
/*
  Copyright (c) 2006 Andres Colubri

  This source is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This code is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  A copy of the GNU General Public License is available on the World
  Wide Web at <http://www.gnu.org/copyleft/gpl.html>. You can also
  obtain it by writing to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

// Enapsulates a movable and resizable filter.
class FX
{
    FX()
    {
        params = new GLTextureFilterParams();
        params.parFlt1 = 30;
        enabled = false;
    }
    
    boolean active(float t)
    {
        return enabled && (t0 <= t) && (t < t1);
    }
    
    void setParams()
    {
          
    }
    
    void apply(GLTexture tex0, GLTexture tex1)
    {
        tex0.filter(texFilter, tex1, params);
    }              
    
    void activate(float x, float y, float w, float h, float t0, float t1, GLTextureFilter texFilter)
    {
        this.x = x;
        this.y = y;
        this.w = w;
        this.h = h;
        this.t0 = t0;
        this.t1 = t1;
        
        x0 = x;
        y0 = y;
        w0 = w;
        h0 = h;        
        
        this.texFilter = texFilter;
        enabled = true;      
    }
    
    void activate(float x, float y, float w, float h, float t0, float t1)
    {
        this.x = x;
        this.y = y;
        this.w = w;
        this.h = h;
        this.t0 = t0;
        this.t1 = t1;
        
        x0 = x;
        y0 = y;
        w0 = w;
        h0 = h;        
        
        enabled = true;      
    }
    
    void desactivate()
    {
        enabled = false;  
    }
    
    void draw()
    {
        if (enabled) 
        {
            fill(206, 46, 46);  
            stroke(0);
            rect(x, y, w, h);

            fill(255);            
            ellipse(x + 0.5 * w, y + 0.5 * h, 0.5 * h, 0.5 * h);
            
            triangle(x, y, x + 5, y + 0.5 * h, x, y + h);
            triangle(x + w - 5, y + 0.5 * h, x + w, y, x + w, y + h);            
        }
    }
    
    void update(float mX, float mY, float mX0, float mY0)
    {
        if (!((y <= mY) && (mY <= y + h) && (y <= mY0) && (mY0 <= y + h))) return;
      
        if ((x <= mX0) && (mX0 <= x + 5)) updateLeft(mX, mX0);
        if ((x + w - 5 <= mX0) && (mX0 <= x + w)) updateRight(mX, mX0);
      
        if ((x + 0.5 * (w - h) <= mX0) && (mX0 <= x + 0.5 * (w + h))) updateCenter(mX, mX0);
  
        ConstrainBorders();  
    }
    
    void updateLeft(float mX, float mX0)
    {
        float dX = mX - mX0;
        x += dX;
        w -= dX;     
    }
    
    void updateRight(float mX, float mX0)
    {
        float dX = mX - mX0;
        w += dX;
    }    
    
    void updateCenter(float mX, float mX0)
    {
        float dX = mX - mX0;
        x += dX;
    }      

    void ConstrainBorders()
    {
        x = constrain(x, x0, x0 + w0);
        w = constrain(w, 0 , x0 + w0 - x);
    
        t0 = (x - x0) / w0;
        t1 = t0 + w / w0;
        println(t0 + " " + t1);
    }
    
    boolean enabled;
    
    float x, y, w, h;
    float x0, y0, w0, h0;    
    float t0, t1;
    
    GLTextureFilter texFilter;
    GLTextureFilterParams params;  
}
