/**
 * Realmatik
 * by Andres Colubri. April 2008.
 * 
 * This program is a propotype of a real-time video filter application..
 *
 */
 
/*
  Copyright (c) 2008 Andres Colubri

  This source is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This code is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  A copy of the GNU General Public License is available on the World
  Wide Web at <http://www.gnu.org/copyleft/gpl.html>. You can also
  obtain it by writing to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

import processing.opengl.*;

import codeanticode.gsvideo.*;
import codeanticode.gltexture.*;

import sojamo.drop.*;
import controlP5.*;

GSMovie movie;

int numFX = 5;
GLTexture tex[];
FX fx[];

GLTextureFilter pixel, gauss, emboss, poster, edge;

ControlP5 controlP5;
SDrop drop;
Slider timelineSlider;
Toggle playToggle;
MultiList fxMenu;
boolean play;

PFont font;

void setup()
{
    size(640, 480, OPENGL);
    drop = new SDrop(this);    
    
    controlP5 = new ControlP5(this);
    controlP5.setAutoDraw(false);

    movie = null;
    
    font = loadFont("Miriam-28.vlw");
    textFont(font, 28);
   
    tex = new GLTexture[numFX + 1];
    fx = new FX[numFX];
    for (int i = 0; i < numFX; i++)
    {
        tex[i] = new GLTexture(this);
        fx[i] = new FX();        
    }
    tex[numFX] = new GLTexture(this);

   
    gauss = new GLTextureFilter(this, "gaussBlur.xml");
    pixel = new GLTextureFilter(this, "pixelate.xml");
    emboss = new GLTextureFilter(this, "pulsatingEmboss.xml");
    poster = new GLTextureFilter(this, "posterize.xml");
    edge = new GLTextureFilter(this, "edgeDetect.xml");
    
    fx[0].activate(50, 280, 550, 20, 0.0, 1.0, gauss);
    
    timelineSlider = controlP5.addSlider("timeline", 
                                         0, 1, 0, // Minimum, maximum, initial value
                                         50, 250, 550, 20); // x0, y0, x1, y1   
    timelineSlider.setLabel("");    
    timelineSlider.update();
    
    playToggle = controlP5.addToggle("play", false, 610, 250, 20, 20);
    
    fxMenu = controlP5.addMultiList("fxMenu", 0, 280, 40, 20);
    MultiListButton b;
    b = fxMenu.add("FX1", 1);
    // add items to a sublist of button "FX1"
    b.add("FX11", 11).setLabel("gauss");
    b.add("FX12", 12).setLabel("emboss");    
    b.add("FX13", 13).setLabel("edge");
    b.add("FX14", 14).setLabel("poster");
    b.add("FX15", 15).setLabel("pixel");
    b.add("FX16", 16).setLabel("no fx");  
  
    b = fxMenu.add("FX2", 1);
    // add items to a sublist of button "FX2"
    b.add("FX21", 21).setLabel("gauss");
    b.add("FX22", 22).setLabel("emboss");    
    b.add("FX23", 23).setLabel("edge");
    b.add("FX24", 24).setLabel("poster");
    b.add("FX25", 25).setLabel("pixel");
    b.add("FX26", 26).setLabel("no fx");
    
    b = fxMenu.add("FX3", 1);
    // add items to a sublist of button "FX3"
    b.add("FX31", 31).setLabel("gauss");
    b.add("FX32", 32).setLabel("emboss");    
    b.add("FX33", 33).setLabel("edge");
    b.add("FX34", 34).setLabel("poster");
    b.add("FX35", 35).setLabel("pixel");
    b.add("FX36", 36).setLabel("no fx"); 

    b = fxMenu.add("FX4", 1);
    // add items to a sublist of button "FX4"
    b.add("FX41", 41).setLabel("gauss");
    b.add("FX42", 42).setLabel("emboss");    
    b.add("FX43", 43).setLabel("edge");
    b.add("FX44", 44).setLabel("poster");
    b.add("FX45", 45).setLabel("pixel");
    b.add("FX46", 46).setLabel("no fx");
    
    b = fxMenu.add("FX5", 1);
    // add items to a sublist of button "FX5"
    b.add("FX51", 51).setLabel("gauss");
    b.add("FX52", 52).setLabel("emboss");    
    b.add("FX53", 53).setLabel("edge");
    b.add("FX54", 54).setLabel("poster");
    b.add("FX55", 55).setLabel("pixel");
    b.add("FX56", 56).setLabel("no fx");
}

void draw()
{
    if ((movie == null) || (movie.width == 1) || (movie.height == 1)) 
    {
        background(0);
        fill(255);
        text("Drag and drop a movie file here.", 140, 130);
    }
    else
    {
        float f = movie.time() / movie.duration();
        
        tex[0].putPixelsIntoTexture(movie);
        
        int k = 0;
        for (int i = 0; i < numFX; i++)
            if (fx[i].active(f))
            {
                fx[i].setParams();
                fx[i].apply(tex[k], tex[k+1]);
                k++;
            }
        
        tex[0].renderTexture(0, 0, 320, 240);
        tex[k].renderTexture(320, 0, 320, 240);

        timelineSlider.setValue(f);
    }
    
    drawFX();
    controlP5.draw();        
}

void drawFX()
{
    fill(0);
    stroke(0);
    rect(40, 280, 600, 200);
    
    for (int i = 0; i < numFX; i++) fx[i].draw();
}

void dropEvent(DropEvent theDropEvent) 
{
  if ((movie == null) && theDropEvent.isFile())
  {
      String fn = theDropEvent.file().getPath();
      println(fn);
      movie = new GSMovie(this, fn);
      playToggle.setState(true);
      movie.loop();
      play = true;
      timelineSlider.setValue(0);     
  }
}

void movieEvent(GSMovie movie)
{
    movie.read();
}

void controlEvent(ControlEvent event) 
{
    if (event.controller().name() == "timeline")
    {
        float t  = event.controller().value() * movie.duration();
        if (0.1 < abs(t - movie.time()))
        {
            if (!play) movie.play();
            movie.jump(t);
            if (!play) movie.pause();
        }
    } 
    else if (event.controller().name().substring(0, 2).equals("FX"))
    {
        for (int i = 0; i < numFX; i++)
        {
            int n = int(event.value());
            int m0 = 10 * (i + 1);
            int m1 = 10 * (i + 2);
            if ((m0 <= n) && (n < m1))
            {
                int j = n - m0;
                
                if (j == 1) fx[i].texFilter = gauss;
                else if (j == 2) fx[i].texFilter = emboss;
                else if (j == 3) fx[i].texFilter = edge;                
                else if (j == 4) fx[i].texFilter = poster;                
                else if (j == 5) fx[i].texFilter = pixel;                
                else 
                {
                    fx[i].desactivate();  
                    return;
                }
                if (!fx[i].enabled) fx[i].activate(50, 280 + 20 * i, 550, 20, 0.0, 1.0);
            }
        }
    }
}

void play(boolean theFlag) 
{
    if (theFlag == true) 
    {
        movie.loop(); 
        play = true;         
    } 
    else 
    {
        movie.pause();
        play = false;        
    }
}

void mouseDragged()
{
    for (int i = 0; i < numFX; i++) fx[i].update(mouseX, mouseY, pmouseX, pmouseY);  
}
